"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _eventstoreBase = require("@resolve-js/eventstore-base");

var _constants = require("./constants");

const makeThreadArrayQuery = vectorConditions => {
  return `${vectorConditions.map((threadCounter, threadId) => `"threadId" = ${threadId} AND "threadCounter" >= ${threadCounter}::${_constants.INT8_SQL_TYPE} `).join(' OR ')}`;
};

const createLoadQuery = ({
  escapeId,
  escape,
  eventsTableName,
  databaseName
}, {
  eventTypes,
  aggregateIds,
  cursor,
  limit
}) => {
  const vectorConditions = (0, _eventstoreBase.cursorToThreadArray)(cursor);

  const injectString = value => `${escape(value)}`;

  const queryConditions = ['1 = 1'];

  if (eventTypes != null) {
    if (eventTypes.length === 0) {
      return null;
    }

    queryConditions.push(`"type" IN (${eventTypes.map(injectString)})`);
  }

  if (aggregateIds != null) {
    if (aggregateIds.length === 0) {
      return null;
    }

    queryConditions.push(`"aggregateId" IN (${aggregateIds.map(injectString)})`);
  }

  const resultQueryCondition = queryConditions.join(' AND ');
  const databaseNameAsId = escapeId(databaseName);
  const eventsTableAsId = escapeId(eventsTableName);

  if (limit !== undefined) {
    if (aggregateIds != null && aggregateIds.length === 1) {
      const resultVectorConditions = makeThreadArrayQuery(vectorConditions);
      return [`SELECT * FROM ${databaseNameAsId}.${eventsTableAsId}`, `WHERE (${resultVectorConditions}) AND (${resultQueryCondition})`, `ORDER BY "aggregateVersion" ASC`, `LIMIT ${+limit}`].join('\n');
    }

    return [`SELECT "sortedEvents".* FROM (`, `  SELECT "unitedEvents".* FROM (`, vectorConditions.map((threadCounter, threadId) => `(${[`SELECT * FROM ${databaseNameAsId}.${eventsTableAsId}`, `WHERE (${resultQueryCondition}) AND "threadId" = ${threadId} AND "threadCounter" >= ${threadCounter}::${_constants.INT8_SQL_TYPE}`, `ORDER BY "threadCounter" ASC`, `LIMIT ${+limit}`].join(' ')})`).join(' UNION ALL \n'), `  ) "unitedEvents"`, `  ORDER BY "unitedEvents"."timestamp" ASC`, `  LIMIT ${+limit}`, `) "sortedEvents"`, `ORDER BY "sortedEvents"."timestamp" ASC,`, `"sortedEvents"."threadCounter" ASC,`, `"sortedEvents"."threadId" ASC`].join('\n');
  } else {
    const resultVectorConditions = makeThreadArrayQuery(vectorConditions);
    return [`SELECT * FROM ${databaseNameAsId}.${eventsTableAsId}`, `WHERE (${resultVectorConditions}) AND (${resultQueryCondition})`, `ORDER BY "timestamp" ASC, "threadCounter" ASC, "threadId" ASC`].join('\n');
  }
};

var _default = createLoadQuery;
exports.default = _default;
//# sourceMappingURL=create-load-query.js.map